/* ====================================================================
   Licensed to the Apache Software Foundation (ASF) under one or more
   contributor license agreements.  See the NOTICE file distributed with
   this work for additional information regarding copyright ownership.
   The ASF licenses this file to You under the Apache License, Version 2.0
   (the "License"); you may not use this file except in compliance with
   the License.  You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */

package org.apache.poi.xssf.streaming;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.poi.ss.usermodel.DataFormatter;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.util.SheetUtil;

/**
 * Tracks maximum column width when adding rows to an SXSSF sheet
 * so the column width can be calculated although not all rows are
 * held in memory.
*/
public class AutosizeColumnTracker {
    private SXSSFSheet sheet;
    private int defaultCharWidth = -1;

    private final Set<Integer> monitoredColumns = new HashSet<Integer>();
    private final Map<Integer, Double> maxColumnWidths = new HashMap<Integer, Double>();
    private DataFormatter dataFormatter;

    public AutosizeColumnTracker(SXSSFSheet sheet) {
        this.sheet = sheet;
    }

    public void monitorColumn(int columnIndex) {
        monitoredColumns.add(columnIndex);
    }

    public void trackRow(Row row) {
        trackRow(row, false);
    }

    public void trackRow(Row row, boolean useMergedCells) {

        if (defaultCharWidth < 0) {
            defaultCharWidth = SheetUtil.getDefaultCharWidth(sheet);
        }

        if (dataFormatter == null) {
            dataFormatter = new DataFormatter();
        }

        for (int monitoredColumn : monitoredColumns) {
            final double columnWidth = SheetUtil.getColumnWidthForRow(monitoredColumn, useMergedCells, defaultCharWidth,
                                                                      dataFormatter, row);
            if (!maxColumnWidths.containsKey(monitoredColumn) || columnWidth > maxColumnWidths.get(monitoredColumn)) {
                maxColumnWidths.put(monitoredColumn, columnWidth);
            }
        }
    }

    public void applyAutosizeToColumn(int columnIndex) {
        if (maxColumnWidths.containsKey(columnIndex)) {

            double maxColumnWidth = maxColumnWidths.get(columnIndex);
            maxColumnWidth = Math.min(maxColumnWidth, 255); // The maximum column width for an individual cell is 255 characters
            sheet.setColumnWidth(columnIndex, (int) (maxColumnWidth*256));
        }
    }

}
